"""Split text to sentences.

Use sentence_splitter if supported,
else use polyglot.text.Text
"""
from typing import List, Optional

from tqdm.auto import tqdm
from polyglot.text import Detector, Text
from sentence_splitter import split_text_into_sentences

# fmt: off
# use sentence_splitter if supported
LANG_S = ["ca", "cs", "da", "nl", "en", "fi", "fr", "de",
          "el", "hu", "is", "it", "lv", "lt", "no", "pl",
          "pt", "ro", "ru", "sk", "sl", "es", "sv", "tr"]
# fmt: on


def seg_text(text: str, lang: Optional[str] = None, qmode: bool = False) -> List[str]:
    """
    Split text to sentences.

    Use sentence_splitter if supported,
    else use polyglot.text.Text.

    qmode: skip split_text_into_sentences if True, default False
    """

    if lang is None:
        lang = Detector(text).language.code

    if not qmode and lang in LANG_S:
        _ = []
        for para in tqdm(text.splitlines()):
            if para.strip():
                _.extend(split_text_into_sentences(para, lang))
        return _

        # return split_text_into_sentences(text, lang)

    return [elm.string for elm in Text(text, lang).sentences]



"""Init."""
from pathlib import Path
from .load_text import load_text

__version__ = "0.1.3"

# pylint: disable=invalid-name
cdir = Path(__file__).parent
hlm_zh = load_text(cdir / "340-脂砚斋重批红楼梦.txt")
hlm_en = load_text(cdir / "david-hawks-the-story-of-the-stone.txt")
hlm_en1 = load_text(cdir / "yang-hlm.txt")
hlm_en2 = load_text(cdir / "joly-hlm.txt")

fangfang_en = load_text(cdir / "fangfang-en.txt")
fangfang_zh = load_text(cdir / "fangfang-zh.txt")



"""Encode sents in batch of size b_size."""
from typing import Callable, Optional

from tqdm.auto import trange
import numpy as np
from logzero import logger

# embed: Any = None


# fmt: off
def encoder(
        data: list,
        b_size: int = 100,
        embed: Optional[Callable] = None,
) -> np.ndarray:
    # fmt: on
    """Encode sents in batch of size b_size.

    >>> assert encoder(['test']).shape == (1, 512)
    """
    if embed is None:
        logger.warning(" embed is None, return empty np.ndarray.")
        return np.ndarray([0])

    b_size = 100
    len_ = len(data)
    # quo, rem = divmod(len_, b_size)
    quo, _ = divmod(len_, b_size)

    # abcnews 1082168, ~9 hours in duanzi
    # misinformation abstract, 5501, ~30 min
    encoded_data: Optional[np.ndarray] = None

    # quo part
    for i in trange(quo):
        _ = data[i * b_size: (i + 1) * b_size]
        _ = np.array([elm for elm in embed(_)]).astype("float32")
        if encoded_data is None:
            encoded_data = _
        else:
            encoded_data = np.concatenate((encoded_data, _), axis=0)

    # rem part
    _ = data[quo * b_size:]
    _ = np.array([elm for elm in embed(_)]).astype("float32")
    if encoded_data is None:
        encoded_data = _
    else:
        encoded_data = np.concatenate((encoded_data, _), axis=0)

    return encoded_data



"""Load text with best effort to determine encoding."""

from typing import Union
from pathlib import Path
import re

import cchardet
from logzero import logger


def load_text(filename: Union[str, Path]) -> str:
    """Load text for given filepath."""
    if not Path(filename).is_file():
        raise SystemExit(
            f"{Path(filename).resolve().as_posix()} does not exist or is not a file"
        )
    try:
        _ = cchardet.detect(Path(filename).read_bytes())
    except Exception as exc:
        logger.error("cchardet exc: %s, setting encoding to utf8", exc)
        _ = {"encoding": "utf8"}
    encoding = _["encoding"]

    try:
        cont = Path(filename).read_text(encoding)
    except Exception as exc:
        logger.error(" read_text exc: %s", exc)
        raise

    # replace unicode spaces with normal space " "
    cont = re.sub(r"[\u3000]", " ", cont)

    _ = [elm.strip() for elm in cont.splitlines() if elm.strip()]
    return "\n".join(_)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['hlm_texts']

package_data = \
{'': ['*']}

install_requires = \
['Morfessor>=2.0.6,<3.0.0',
 'cchardet>=2.1.7,<3.0.0',
 'halo>=0.0.31,<0.0.32',
 'logzero>=1.6.3,<2.0.0',
 'polyglot>=16.7.4,<17.0.0',
 'sentence_splitter>=1.4,<2.0',
 'tqdm>=4.54.1,<5.0.0']

setup_kwargs = {
    'name': 'hlm-texts',
    'version': '0.1.3a0',
    'description': '',
    'long_description': "# hlm-texts [![Codacy Badge](https://api.codacy.com/project/badge/Grade/31c6bcb6723942a3bb12474cd7e74dac)](https://app.codacy.com/gh/ffreemt/hlm-texts?utm_source=github.com&utm_medium=referral&utm_content=ffreemt/hlm-texts&utm_campaign=Badge_Grade)[![Code style: black](https://img.shields.io/badge/code%20style-black-000000.svg)](https://github.com/psf/black)[![License: MIT](https://img.shields.io/badge/License-MIT-yellow.svg)](https://opensource.org/licenses/MIT)[![PyPI version](https://badge.fury.io/py/hlm-texts.svg)](https://badge.fury.io/py/hlm-texts)\nHlm at your fingertips\n\nThis repo currently just has `340-脂砚斋重批红楼梦.txt` and `david-hawks-the-story-of-the-stone.txt`. It may expand to other versions. If you wish to have one particular version included, make a `pull request` (fork, upload files and click the pull request button) or provide a text file to me.\n\n## Installation\n`pip install hlm-texts`\n\n## Usage\n\n```python\nfrom hlm_texts import hlm_en, hlm_zh\n```\n\n`hlm_zh` and `hlm_en` are copies of `340-脂砚斋重批红楼梦.txt` and `david-hawks-the-story-of-the-stone.txt`, respectively, with blank lines removed and paragraphs retaind.\n\n## Notes\nThe repo is for study purpose only. If you believe that your interests have been violated in any way, please let me know. I'll promptly follow it up with appropriate actions.",
    'author': 'freemt',
    'author_email': None,
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/ffreemt/hlm-texts',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
